@@ -1,7 +1,7 @@
 /*
- *  Copyright (C) 2014-2016 Savoir-faire Linux Inc.
+ *  Copyright (C) 2014-2017 Savoir-faire Linux Inc.
  *  Author(s) : Adrien Béraud <adrien.beraud@savoirfairelinux.com>
- *              Simon Désaulniers <sim.desaulniers@gmail.com>
+ *              Simon Désaulniers <simon.desaulniers@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #pragma once
@@ -41,7 +41,15 @@
 
 namespace dht {
 
+namespace net {
 struct Request;
+} /* namespace net */
+
+struct Storage;
+struct ValueStorage;
+class StorageBucket;
+struct Listener;
+struct LocalListener;
 
 /**
  * Main Dht class.
@@ -50,7 +58,7 @@ struct Request;
  * Must be given open UDP sockets and ::periodic must be
  * called regularly.
  */
-class Dht {
+class OPENDHT_PUBLIC Dht {
 public:
 
     // [[deprecated]]
@@ -160,8 +168,8 @@ public:
      * @param key the key for which to query data for.
      * @param cb a function called when new values are found on the network.
      *           It should return false to stop the operation.
-     * @param donecb a function called when the operation is complete.
-                     cb and donecb won't be called again afterward.
+     * @param done_cb a function called when the operation is complete.
+                     cb and done_cb won't be called again afterward.
      * @param q a query used to filter values on the remotes before they send a
      *          response.
      */
@@ -173,12 +181,12 @@ public:
     /**
      * Get locally stored data for the given hash.
      */
-    std::vector<std::shared_ptr<Value>> getLocal(const InfoHash& key, Value::Filter f = Value::AllFilter()) const;
+    std::vector<Sp<Value>> getLocal(const InfoHash& key, Value::Filter f = Value::AllFilter()) const;
 
     /**
      * Get locally stored data for the given key and value id.
      */
-    std::shared_ptr<Value> getLocalById(const InfoHash& key, Value::Id vid) const;
+    Sp<Value> getLocalById(const InfoHash& key, Value::Id vid) const;
 
     /**
      * Announce a value on all available protocols (IPv4, IPv6).
@@ -187,12 +195,12 @@ public:
      * The done callback will be called once, when the first announce succeeds, or fails.
      */
     void put(const InfoHash& key,
-            std::shared_ptr<Value>,
+            Sp<Value>,
             DoneCallback cb=nullptr,
             time_point created=time_point::max(),
             bool permanent = false);
     void put(const InfoHash& key,
-            const std::shared_ptr<Value>& v,
+            const Sp<Value>& v,
             DoneCallbackSimple cb,
             time_point created=time_point::max(),
             bool permanent = false)
@@ -220,12 +228,12 @@ public:
     /**
      * Get data currently being put at the given hash.
      */
-    std::vector<std::shared_ptr<Value>> getPut(const InfoHash&);
+    std::vector<Sp<Value>> getPut(const InfoHash&);
 
     /**
      * Get data currently being put at the given hash with the given id.
      */
-    std::shared_ptr<Value> getPut(const InfoHash&, const Value::Id&);
+    Sp<Value> getPut(const InfoHash&, const Value::Id&);
 
     /**
      * Stop any put/announce operation at the given location,
@@ -268,8 +276,8 @@ public:
     std::vector<ValuesExport> exportValues() const;
     void importValues(const std::vector<ValuesExport>&);
 
-    unsigned getNodesStats(sa_family_t af, unsigned *good_return, unsigned *dubious_return, unsigned *cached_return,
-            unsigned *incoming_return) const;
+    NodeStats getNodesStats(sa_family_t af) const;
+
     std::string getStorageLog() const;
     std::string getStorageLog(const InfoHash&) const;
 
@@ -327,14 +335,11 @@ private:
     /* Number of listening nodes */
     static constexpr unsigned LISTEN_NODES {4};
 
-    /* The maximum number of values we store for a given hash. */
-    static constexpr unsigned MAX_VALUES {1024};
-
     /* The maximum number of hashes we're willing to track. */
     static constexpr unsigned MAX_HASHES {16384};
 
     /* The maximum number of searches we keep data about. */
-    static constexpr unsigned MAX_SEARCHES {2048};
+    static constexpr unsigned MAX_SEARCHES {16384};
 
     static constexpr std::chrono::minutes MAX_STORAGE_MAINTENANCE_EXPIRE_TIME {10};
 
@@ -346,17 +351,13 @@ private:
 
     static constexpr std::chrono::seconds REANNOUNCE_MARGIN {10};
 
-    static constexpr size_t TOKEN_SIZE {64};
+    static constexpr size_t TOKEN_SIZE {32};
 
     // internal structures
     struct SearchNode;
     struct Get;
     struct Announce;
-    struct LocalListener;
     struct Search;
-    struct ValueStorage;
-    struct Listener;
-    struct Storage;
 
     // prevent copy
     Dht(const Dht&) = delete;
@@ -364,8 +365,8 @@ private:
 
     InfoHash myid {};
 
-    std::array<uint8_t, 8> secret {{}};
-    std::array<uint8_t, 8> oldsecret {{}};
+    uint64_t secret {};
+    uint64_t oldsecret {};
 
     // registred types
     std::map<ValueType::Id, ValueType> types;
@@ -375,17 +376,19 @@ private:
     //       Only nodes running only as bootstrap nodes should
     //       be put in bootstrap mode.
     const bool is_bootstrap {false};
+    const bool maintain_storage {false};
 
     // the stuff
     RoutingTable buckets4 {};
     RoutingTable buckets6 {};
 
     std::map<InfoHash, Storage> store;
+    std::map<SockAddr, StorageBucket, SockAddr::ipCmp> store_quota;
     size_t total_values {0};
     size_t total_store_size {0};
     size_t max_store_size {DEFAULT_STORAGE_LIMIT};
 
-    using SearchMap = std::map<InfoHash, std::shared_ptr<Search>>;
+    using SearchMap = std::map<InfoHash, Sp<Search>>;
     SearchMap searches4 {};
     SearchMap searches6 {};
     uint16_t search_id {0};
@@ -397,28 +400,32 @@ private:
 
     // timing
     Scheduler scheduler;
-    std::shared_ptr<Scheduler::Job> nextNodesConfirmation {};
-    std::shared_ptr<Scheduler::Job> nextStorageMaintenance {};
-    time_point mybucket_grow_time {time_point::min()}, mybucket6_grow_time {time_point::min()};
+    Sp<Scheduler::Job> nextNodesConfirmation {};
+    Sp<Scheduler::Job> nextStorageMaintenance {};
 
-    NetworkEngine network_engine;
+    net::NetworkEngine network_engine;
     unsigned pending_pings4 {0};
     unsigned pending_pings6 {0};
 
     using ReportedAddr = std::pair<unsigned, SockAddr>;
     std::vector<ReportedAddr> reported_addr;
 
+    std::mt19937_64 rd {crypto::getSeededRandomEngine<std::mt19937_64>()};
+
     void rotateSecrets();
 
-    Blob makeToken(const sockaddr *sa, bool old) const;
-    bool tokenMatch(const Blob& token, const sockaddr *sa) const;
+    Blob makeToken(const SockAddr&, bool old) const;
+    bool tokenMatch(const Blob& token, const SockAddr&) const;
 
     void reportedAddr(const SockAddr&);
 
     // Storage
-    void storageAddListener(const InfoHash& id, const std::shared_ptr<Node>& node, size_t tid, Query&& = {});
-    bool storageStore(const InfoHash& id, const std::shared_ptr<Value>& value, time_point created);
-    void expireStorage();
+    void storageAddListener(const InfoHash& id, const Sp<Node>& node, size_t tid, Query&& = {});
+    bool storageStore(const InfoHash& id, const Sp<Value>& value, time_point created, const SockAddr& sa = {});
+    void expireStore();
+    void expireStorage(InfoHash h);
+    void expireStore(decltype(store)::iterator);
+
     void storageChanged(const InfoHash& id, Storage& st, ValueStorage&);
     std::string printStorageLog(const decltype(store)::value_type&) const;
 
@@ -451,15 +458,14 @@ private:
     }
 
     void expireBuckets(RoutingTable&);
-    int sendCachedPing(Bucket& b);
+    void sendCachedPing(Bucket& b);
     bool bucketMaintenance(RoutingTable&);
     void dumpBucket(const Bucket& b, std::ostream& out) const;
 
     // Nodes
-    void onNewNode(const std::shared_ptr<Node>& node, int confirm);
-    std::shared_ptr<Node> findNode(const InfoHash& id, sa_family_t af);
-    const std::shared_ptr<Node> findNode(const InfoHash& id, sa_family_t af) const;
-    bool trySearchInsert(const std::shared_ptr<Node>& node);
+    void onNewNode(const Sp<Node>& node, int confirm);
+    const Sp<Node> findNode(const InfoHash& id, sa_family_t af) const;
+    bool trySearchInsert(const Sp<Node>& node);
 
     // Searches
 
@@ -470,10 +476,10 @@ private:
      * Low-level method that will perform a search on the DHT for the specified
      * infohash (id), using the specified IP version (IPv4 or IPv6).
      */
-    std::shared_ptr<Search> search(const InfoHash& id, sa_family_t af, GetCallback = {}, QueryCallback = {}, DoneCallback = {}, Value::Filter = {}, Query q = {});
+    Sp<Search> search(const InfoHash& id, sa_family_t af, GetCallback = {}, QueryCallback = {}, DoneCallback = {}, Value::Filter = {}, Query q = {});
 
-    void announce(const InfoHash& id, sa_family_t af, std::shared_ptr<Value> value, DoneCallback callback, time_point created=time_point::max(), bool permanent = false);
-    size_t listenTo(const InfoHash& id, sa_family_t af, GetCallback cb, Value::Filter f = Value::AllFilter(), const std::shared_ptr<Query>& q = {});
+    void announce(const InfoHash& id, sa_family_t af, Sp<Value> value, DoneCallback callback, time_point created=time_point::max(), bool permanent = false);
+    size_t listenTo(const InfoHash& id, sa_family_t af, GetCallback cb, Value::Filter f = Value::AllFilter(), const Sp<Query>& q = {});
 
     /**
      * Refill the search with good nodes if possible.
@@ -483,8 +489,6 @@ private:
      * @return the number inserted nodes.
      */
     unsigned refill(Search& sr);
-
-    Search *findSearch(unsigned short tid, sa_family_t af);
     void expireSearches();
 
     void confirmNodes();
@@ -498,10 +502,10 @@ private:
      * @param ws      A weak pointer to the search concerned by the request.
      * @param query   The query sent to the node.
      */
-    void searchNodeGetDone(const Request& status,
-            NetworkEngine::RequestAnswer&& answer,
+    void searchNodeGetDone(const net::Request& status,
+            net::RequestAnswer&& answer,
             std::weak_ptr<Search> ws,
-            std::shared_ptr<Query> query);
+            Sp<Query> query);
 
     /**
      * Generic function to execute when a 'get' request expires.
@@ -512,7 +516,7 @@ private:
      * @param ws      A weak pointer to the search concerned by the request.
      * @param query   The query sent to the node.
      */
-    void searchNodeGetExpired(const Request& status, bool over, std::weak_ptr<Search> ws, std::shared_ptr<Query> query);
+    void searchNodeGetExpired(const net::Request& status, bool over, std::weak_ptr<Search> ws, Sp<Query> query);
 
     /**
      * This method recovers sends individual request for values per id.
@@ -521,12 +525,12 @@ private:
      * @param query  The initial query passed through the API.
      * @param n      The node to which send the requests.
      */
-    void paginate(std::weak_ptr<Search> ws, std::shared_ptr<Query> query, SearchNode* n);
+    void paginate(std::weak_ptr<Search> ws, Sp<Query> query, SearchNode* n);
 
     /**
      * If update is true, this method will also send message to synced but non-updated search nodes.
      */
-    SearchNode* searchSendGetValues(std::shared_ptr<Search> sr, SearchNode *n = nullptr, bool update = true);
+    SearchNode* searchSendGetValues(Sp<Search> sr, SearchNode *n = nullptr, bool update = true);
 
     /**
      * Forwards an 'announce' request for a list of nodes to the network engine.
@@ -534,7 +538,7 @@ private:
      * @param sr  The search for which we want to announce a value.
      * @param announce  The 'announce' structure.
      */
-    void searchSendAnnounceValue(const std::shared_ptr<Search>& sr);
+    void searchSendAnnounceValue(const Sp<Search>& sr);
 
     /**
      * Main process of a Search's operations. This function will demand the
@@ -543,51 +547,52 @@ private:
      *
      * @param sr  The search to execute its operations.
      */
-    void searchStep(std::shared_ptr<Search> sr);
+    void searchStep(Sp<Search> sr);
     void dumpSearch(const Search& sr, std::ostream& out) const;
 
     bool neighbourhoodMaintenance(RoutingTable&);
 
-    void processMessage(const uint8_t *buf, size_t buflen, const SockAddr&);
-
-    void onError(std::shared_ptr<Request> node, DhtProtocolException e);
+    void onError(Sp<net::Request> node, net::DhtProtocolException e);
     /* when our address is reported by a distant peer. */
     void onReportedAddr(const InfoHash& id, const SockAddr&);
     /* when we receive a ping request */
-    NetworkEngine::RequestAnswer onPing(std::shared_ptr<Node> node);
+    net::RequestAnswer onPing(Sp<Node> node);
     /* when we receive a "find node" request */
-    NetworkEngine::RequestAnswer onFindNode(std::shared_ptr<Node> node, const InfoHash& hash, want_t want);
-    void onFindNodeDone(const Request& status, NetworkEngine::RequestAnswer& a, std::shared_ptr<Search> sr);
+    net::RequestAnswer onFindNode(Sp<Node> node, const InfoHash& hash, want_t want);
+    void onFindNodeDone(const Sp<Node>& status,
+            net::RequestAnswer& a,
+            Sp<Search> sr);
     /* when we receive a "get values" request */
-    NetworkEngine::RequestAnswer onGetValues(std::shared_ptr<Node> node,
+    net::RequestAnswer onGetValues(Sp<Node> node,
             const InfoHash& hash,
             want_t want,
             const Query& q);
-    void onGetValuesDone(const Request& status,
-            NetworkEngine::RequestAnswer& a,
-            std::shared_ptr<Search>& sr,
-            const std::shared_ptr<Query>& orig_query);
+    void onGetValuesDone(const Sp<Node>& status,
+            net::RequestAnswer& a,
+            Sp<Search>& sr,
+            const Sp<Query>& orig_query);
     /* when we receive a listen request */
-    NetworkEngine::RequestAnswer onListen(std::shared_ptr<Node> node,
+    net::RequestAnswer onListen(Sp<Node> node,
             const InfoHash& hash,
             const Blob& token,
-            size_t rid,
+            size_t socket_id,
             const Query& query);
-    void onListenDone(const Request& status,
-            NetworkEngine::RequestAnswer& a,
-            std::shared_ptr<Search>& sr,
-            const std::shared_ptr<Query>& orig_query);
+    void onListenDone(const Sp<Node>& status,
+            net::RequestAnswer& a,
+            Sp<Search>& sr);
     /* when we receive an announce request */
-    NetworkEngine::RequestAnswer onAnnounce(std::shared_ptr<Node> node,
+    net::RequestAnswer onAnnounce(Sp<Node> node,
             const InfoHash& hash,
             const Blob& token,
-            const std::vector<std::shared_ptr<Value>>& v,
+            const std::vector<Sp<Value>>& v,
             const time_point& created);
-    NetworkEngine::RequestAnswer onRefresh(std::shared_ptr<Node> node,
+    net::RequestAnswer onRefresh(Sp<Node> node,
             const InfoHash& hash,
             const Blob& token,
             const Value::Id& vid);
-    void onAnnounceDone(const Request& status, NetworkEngine::RequestAnswer& a, std::shared_ptr<Search>& sr);
+    void onAnnounceDone(const Sp<Node>& status,
+            net::RequestAnswer& a,
+            Sp<Search>& sr);
 };
 
 }
