@@ -1,7 +1,7 @@
 /*
- *  Copyright (C) 2014-2016 Savoir-faire Linux Inc.
+ *  Copyright (C) 2014-2017 Savoir-faire Linux Inc.
  *  Author(s) : Adrien Béraud <adrien.beraud@savoirfairelinux.com>
- *              Simon Désaulniers <sim.desaulniers@gmail.com>
+ *              Simon Désaulniers <simon.desaulniers@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
+ *  along with this program. If not, see <https://www.gnu.org/licenses/>.
  */
 
 #pragma once
@@ -23,6 +22,7 @@
 #include "infohash.h"
 #include "crypto.h"
 #include "utils.h"
+#include "sockaddr.h"
 
 #include <msgpack.hpp>
 
@@ -37,6 +37,10 @@
 #include <chrono>
 #include <set>
 
+#if OPENDHT_PROXY_SERVER
+#include <json/json.h>
+#endif //OPENDHT_PROXY_SERVER
+
 namespace dht {
 
 struct Value;
@@ -52,7 +56,7 @@ struct Query;
  * @param form_addr: network address of the incoming request.
  * @param from_len: network address lendth of the incoming request.
  */
-using StorePolicy = std::function<bool(InfoHash key, std::shared_ptr<Value>& value, InfoHash from, const sockaddr* from_addr, socklen_t from_len)>;
+using StorePolicy = std::function<bool(InfoHash key, std::shared_ptr<Value>& value, const InfoHash& from, const SockAddr& addr)>;
 
 /**
  * An edition policy is applied once to every incoming value storage requests,
@@ -68,15 +72,15 @@ using StorePolicy = std::function<bool(InfoHash key, std::shared_ptr<Value>& val
  * @param form_addr: network address of the incoming request.
  * @param from_len: network address lendth of the incoming request.
  */
-using EditPolicy = std::function<bool(InfoHash key, const std::shared_ptr<Value>& old_val, std::shared_ptr<Value>& new_val, InfoHash from, const sockaddr* from_addr, socklen_t from_len)>;
+using EditPolicy = std::function<bool(InfoHash key, const std::shared_ptr<Value>& old_val, std::shared_ptr<Value>& new_val, const InfoHash& from, const SockAddr& addr)>;
 
 static constexpr const size_t MAX_VALUE_SIZE {1024 * 64};
 
-struct ValueType {
+struct OPENDHT_PUBLIC ValueType {
     typedef uint16_t Id;
 
-    static bool DEFAULT_STORE_POLICY(InfoHash, std::shared_ptr<Value>& v, InfoHash, const sockaddr*, socklen_t);
-    static bool DEFAULT_EDIT_POLICY(InfoHash, const std::shared_ptr<Value>&, std::shared_ptr<Value>&, InfoHash, const sockaddr*, socklen_t) {
+    static bool DEFAULT_STORE_POLICY(InfoHash, std::shared_ptr<Value>& v, const InfoHash&, const SockAddr&);
+    static bool DEFAULT_EDIT_POLICY(InfoHash, const std::shared_ptr<Value>&, std::shared_ptr<Value>&, const InfoHash&, const SockAddr&) {
         return false;
     }
 
@@ -114,7 +118,7 @@ struct ValueType {
  * Values are stored at a given InfoHash in the Dht, but also have a
  * unique ID to distinguish between values stored at the same location.
  */
-struct Value
+struct OPENDHT_PUBLIC Value
 {
     enum class Field : int {
         None = 0,
@@ -346,6 +350,14 @@ struct Value
     Value(ValueType::Id t, const uint8_t* dat_ptr, size_t dat_len, Id id = INVALID_ID)
      : id(id), type(t), data(dat_ptr, dat_ptr+dat_len) {}
 
+#if OPENDHT_PROXY_SERVER
+    /**
+     * Build a value from a json object
+     * @param json
+     */
+    Value(Json::Value& json);
+#endif //OPENDHT_PROXY_SERVER
+
     template <typename Type>
     Value(ValueType::Id t, const Type& d, Id id = INVALID_ID)
      : id(id), type(t), data(packMsg(d)) {}
@@ -409,7 +421,7 @@ struct Value
     }
 
     /** print value for debugging */
-    friend std::ostream& operator<< (std::ostream& s, const Value& v);
+    OPENDHT_PUBLIC friend std::ostream& operator<< (std::ostream& s, const Value& v);
 
     std::string toString() const {
         std::stringstream ss;
@@ -417,6 +429,18 @@ struct Value
         return ss.str();
     }
 
+#if OPENDHT_PROXY_SERVER
+    /**
+     * Build a json object from a value
+     * Example:
+     * {
+     *  "data":"base64ofdata",
+     *   id":"0", "seq":0,"type":3
+     * }
+     */
+    Json::Value toJson() const;
+#endif //OPENDHT_PROXY_SERVER
+
     /** Return the size in bytes used by this value in memory (minimum). */
     size_t size() const;
 
@@ -424,11 +448,11 @@ struct Value
     void msgpack_pack_to_sign(Packer& pk) const
     {
         bool has_owner = owner && *owner;
-        pk.pack_map((user_type.empty()?0:1) + (has_owner?(recipient == InfoHash() ? 4 : 5):2));
+        pk.pack_map((user_type.empty()?0:1) + (has_owner?(recipient ? 5 : 4):2));
         if (has_owner) { // isSigned
             pk.pack(std::string("seq"));   pk.pack(seq);
             pk.pack(std::string("owner")); owner->msgpack_pack(pk);
-            if (recipient != InfoHash()) {
+            if (recipient) {
                 pk.pack(std::string("to")); pk.pack(recipient);
             }
         }
@@ -553,7 +577,7 @@ using ValuesExport = std::pair<InfoHash, Blob>;
  * This structure holds the value for a specified field. It's type can either be
  * uint64_t, InfoHash or Blob.
  */
-struct FieldValue
+struct OPENDHT_PUBLIC FieldValue
 {
     FieldValue() {}
     FieldValue(Value::Field f, uint64_t int_value) : field(f), intValue(int_value) {}
@@ -630,30 +654,6 @@ private:
     Blob blobValue {};
 };
 
-
-/**
- * @struct  FieldSelectorDescription
- * @brief   Describes a selection.
- * @details
- * This is meant to narrow data to a set of specified fields. This structure is
- * used to construct a Select structure.
- */
-struct FieldSelectorDescription
-{
-    FieldSelectorDescription() {}
-    FieldSelectorDescription(Value::Field f) : field(f) {}
-
-    Value::Field getField() const { return field; }
-
-    bool operator==(const FieldSelectorDescription& fd) const { return field == fd.field; }
-
-    template <typename Packer>
-    void msgpack_pack(Packer& p) const { p.pack(static_cast<uint8_t>(field)); }
-    void msgpack_unpack(msgpack::object msg) { field = static_cast<Value::Field>(msg.as<int>()); }
-private:
-    Value::Field field {Value::Field::None};
-};
-
 /**
  * @class   Select
  * @brief   Serializable Value field selection.
@@ -661,7 +661,7 @@ private:
  * This is a container for a list of FieldSelectorDescription instances. It
  * describes a complete SELECT query for dht::Value.
  */
-struct Select
+struct OPENDHT_PUBLIC Select
 {
     Select() { }
     Select(const std::string& q_str);
@@ -676,7 +676,8 @@ struct Select
      * @return the resulting Select instance.
      */
     Select& field(Value::Field field) {
-        fieldSelection_.emplace_back(field);
+        if (std::find(fieldSelection_.begin(), fieldSelection_.end(), field) == fieldSelection_.end())
+            fieldSelection_.emplace_back(field);
         return *this;
     }
 
@@ -686,23 +687,24 @@ struct Select
      * @return the set of fields.
      */
     std::set<Value::Field> getSelection() const {
-        std::set<Value::Field> fields {};
-        for (const auto& f : fieldSelection_) {
-            fields.insert(f.getField());
-        }
-        return fields;
+        return {fieldSelection_.begin(), fieldSelection_.end()};
     }
 
     template <typename Packer>
     void msgpack_pack(Packer& pk) const { pk.pack(fieldSelection_); }
     void msgpack_unpack(const msgpack::object& o) {
-        fieldSelection_.clear();
         fieldSelection_ = o.as<decltype(fieldSelection_)>();
     }
 
-    friend std::ostream& operator<<(std::ostream& s, const dht::Select& q);
+    std::string toString() const {
+        std::stringstream ss;
+        ss << *this;
+        return ss.str();
+    }
+
+    OPENDHT_PUBLIC friend std::ostream& operator<<(std::ostream& s, const dht::Select& q);
 private:
-    std::vector<FieldSelectorDescription> fieldSelection_ {};
+    std::vector<Value::Field> fieldSelection_ {};
 };
 
 /**
@@ -712,7 +714,7 @@ private:
  * This is container for a list of FieldValue instances. It describes a
  * complete WHERE query for dht::Value.
  */
-struct Where
+struct OPENDHT_PUBLIC Where
 {
     Where() { }
     Where(const std::string& q_str);
@@ -727,7 +729,9 @@ struct Where
      * @return the resulting Where instance.
      */
     Where& id(Value::Id id) {
-        filters_.emplace_back(Value::Field::Id, id);
+        FieldValue fv {Value::Field::Id, id};
+        if (std::find(filters_.begin(), filters_.end(), fv) == filters_.end())
+            filters_.emplace_back(std::move(fv));
         return *this;
     }
 
@@ -739,7 +743,9 @@ struct Where
      * @return the resulting Where instance.
      */
     Where& valueType(ValueType::Id type) {
-        filters_.emplace_back(Value::Field::ValueType, type);
+        FieldValue fv {Value::Field::ValueType, type};
+        if (std::find(filters_.begin(), filters_.end(), fv) == filters_.end())
+            filters_.emplace_back(std::move(fv));
         return *this;
     }
 
@@ -751,7 +757,9 @@ struct Where
      * @return the resulting Where instance.
      */
     Where& owner(InfoHash owner_pk_hash) {
-        filters_.emplace_back(Value::Field::OwnerPk, owner_pk_hash);
+        FieldValue fv {Value::Field::OwnerPk, owner_pk_hash};
+        if (std::find(filters_.begin(), filters_.end(), fv) == filters_.end())
+            filters_.emplace_back(std::move(fv));
         return *this;
     }
 
@@ -763,7 +771,9 @@ struct Where
      * @return the resulting Where instance.
      */
     Where& seq(uint16_t seq_no) {
-        filters_.emplace_back(Value::Field::SeqNum, seq_no);
+        FieldValue fv {Value::Field::SeqNum, seq_no};
+        if (std::find(filters_.begin(), filters_.end(), fv) == filters_.end())
+            filters_.emplace_back(std::move(fv));
         return *this;
     }
 
@@ -775,7 +785,9 @@ struct Where
      * @return the resulting Where instance.
      */
     Where& userType(std::string user_type) {
-        filters_.emplace_back(Value::Field::UserType, Blob {user_type.begin(), user_type.end()});
+        FieldValue fv {Value::Field::UserType, Blob {user_type.begin(), user_type.end()}};
+        if (std::find(filters_.begin(), filters_.end(), fv) == filters_.end())
+            filters_.emplace_back(std::move(fv));
         return *this;
     }
 
@@ -802,7 +814,13 @@ struct Where
         filters_ = o.as<decltype(filters_)>();
     }
 
-    friend std::ostream& operator<<(std::ostream& s, const dht::Where& q);
+    std::string toString() const {
+        std::stringstream ss;
+        ss << *this;
+        return ss.str();
+    }
+
+    OPENDHT_PUBLIC friend std::ostream& operator<<(std::ostream& s, const dht::Where& q);
 
 private:
     std::vector<FieldValue> filters_;
@@ -816,7 +834,7 @@ private:
  * itselves to include in the peer response to a GET operation. See
  * FieldValue.
  */
-struct Query
+struct OPENDHT_PUBLIC Query
 {
     static const std::string QUERY_PARSE_ERROR;
 
@@ -858,13 +876,13 @@ struct Query
 
     void msgpack_unpack(const msgpack::object& o);
 
-    std::string toString() {
+    std::string toString() const {
         std::stringstream ss;
         ss << *this;
         return ss.str();
     }
 
-    friend std::ostream& operator<<(std::ostream& s, const dht::Query& q) {
+    OPENDHT_PUBLIC friend std::ostream& operator<<(std::ostream& s, const dht::Query& q) {
         return s << "Query[" << q.select << " " << q.where << "]";
     }
 
@@ -880,7 +898,7 @@ struct Query
  * This structures is meant to manipulate a subset of fields normally contained
  * in Value.
  */
-struct FieldValueIndex {
+struct OPENDHT_PUBLIC FieldValueIndex {
     FieldValueIndex() {}
     FieldValueIndex(const Value& v, Select s = {});
     /**
@@ -891,7 +909,7 @@ struct FieldValueIndex {
      */
     bool containedIn(const FieldValueIndex& other) const;
 
-    friend std::ostream& operator<<(std::ostream& os, const FieldValueIndex& fvi);
+    OPENDHT_PUBLIC friend std::ostream& operator<<(std::ostream& os, const FieldValueIndex& fvi);
 
     void msgpack_unpack_fields(const std::set<Value::Field>& fields,
             const msgpack::object& o,
@@ -953,3 +971,5 @@ unpackVector(const std::vector<std::shared_ptr<Value>>& vals) {
 }
 
 }
+
+MSGPACK_ADD_ENUM(dht::Value::Field)
