@@ -76,7 +76,19 @@ struct Dht::SearchNode {
                                                       node is a new candidate for inclusion. */
 
     SearchNode() : node() {}
+    SearchNode(const SearchNode&) = delete;
+    SearchNode(SearchNode&&) = default;
+    SearchNode& operator=(const SearchNode&) = delete;
+    SearchNode& operator=(SearchNode&&) = default;
+
     SearchNode(const Sp<Node>& node) : node(node) {}
+    ~SearchNode() {
+        if (node) {
+            cancelGet();
+            cancelListen();
+            cancelAnnounce();
+        }
+    }
 
     /**
      * Can we use this node to listen/announce now ?
@@ -173,6 +185,15 @@ struct Dht::SearchNode {
         }
     }
 
+    void cancelGet() {
+        for (const auto& status : getStatus) {
+            if (status.second->pending()) {
+                node->cancelRequest(status.second);
+            }
+        }
+        getStatus.clear();
+    }
+
     /**
      * Tells if a request in the status map is expired.
      *
@@ -180,12 +201,12 @@ struct Dht::SearchNode {
      *
      * @return true if there exists an expired request, else false.
      */
-    bool expired(const SyncStatus& status) const {
+    /*static bool expired(const SyncStatus& status) const {
         return std::find_if(status.begin(), status.end(),
             [](const SyncStatus::value_type& r){
                 return r.second and r.second->expired();
             }) != status.end();
-    }
+    }*/
 
     /**
      * Tells if a request in the status map is pending.
@@ -194,19 +215,30 @@ struct Dht::SearchNode {
      *
      * @return true if there exists an expired request, else false.
      */
-    bool pending(const SyncStatus& status) const {
+    static bool pending(const SyncStatus& status) {
         return std::find_if(status.begin(), status.end(),
             [](const SyncStatus::value_type& r){
                 return r.second and r.second->pending();
             }) != status.end();
     }
 
+    bool pendingGet() const { return pending(getStatus); }
+
     bool isAnnounced(Value::Id vid) const {
         auto ack = acked.find(vid);
         if (ack == acked.end() or not ack->second.first)
             return false;
         return ack->second.first->completed();
     }
+    void cancelAnnounce() {
+        for (const auto& status : acked) {
+            const auto& req = status.second.first;
+            if (req and req->pending()) {
+                node->cancelRequest(req);
+            }
+        }
+        acked.clear();
+    }
 
     bool isListening(time_point now) const {
         auto ls = listenStatus.begin();
@@ -229,6 +261,18 @@ struct Dht::SearchNode {
             return false;
         return listen_status->second->reply_time + LISTEN_EXPIRE_TIME > now;
     }
+    void cancelListen() {
+        for (const auto& status : listenStatus)
+            node->cancelRequest(status.second);
+        listenStatus.clear();
+    }
+    void cancelListen(const Sp<Query>& query) {
+        auto it = listenStatus.find(query);
+        if (it != listenStatus.end()) {
+            node->cancelRequest(it->second);
+            listenStatus.erase(it);
+        }
+    }
 
     /**
      * Assumng the node is synced, should a "put" request be sent to this node now ?
@@ -319,7 +363,7 @@ struct Dht::Search {
     unsigned currentlySolicitedNodeCount() const {
         unsigned count = 0;
         for (const auto& n : nodes)
-            if (not n.isBad() and n.pending(n.getStatus))
+            if (not n.isBad() and n.pendingGet())
                 count++;
         return count;
     }
@@ -382,6 +426,21 @@ struct Dht::Search {
     bool isAnnounced(Value::Id id) const;
     bool isListening(time_point now) const;
 
+    void cancelListen(size_t token) {
+        Sp<Query> query;
+        const auto& ll = listeners.find(token);
+        if (ll != listeners.cend()) {
+            query = ll->second.query;
+            listeners.erase(ll);
+        }
+        for (auto& sn : nodes) {
+            if (listeners.empty())
+                sn.cancelListen();
+            else if (query)
+                sn.cancelListen(query);
+        }
+    }
+
     /**
      * @return The number of non-good search nodes.
      */
@@ -430,10 +489,10 @@ struct Dht::Search {
      *
      * @return true if a node has been removed, else false.
      */
-    bool removeExpiredNode(time_point now) {
+    bool removeExpiredNode(const time_point& now) {
         for (auto e = nodes.cend(); e != nodes.cbegin();) {
             const Node& n = *(--e)->node;
-            if (n.isExpired() and n.time + Node::NODE_EXPIRE_TIME < now) {
+            if (n.isRemovable(now)) {
                 //std::cout << "Removing expired node " << n.id << " from IPv" << (af==AF_INET?'4':'6') << " search " << id << std::endl;
                 nodes.erase(e);
                 return true;
@@ -585,7 +644,7 @@ Dht::Search::insertNode(const Sp<Node>& snode, time_point now, const Blob& token
             step_time = time_point::min();
         }
         n = nodes.insert(n, SearchNode(snode));
-        node.time = now;
+        node.setTime(now);
         new_search_node = true;
         if (node.isExpired()) {
             if (not expired)
@@ -672,7 +731,7 @@ Dht::Search::getUpdateTime(time_point now) const
     for (const auto& sn : nodes) {
         if (sn.node->isExpired() or (sn.candidate and t >= TARGET_NODES))
             continue;
-        auto pending = sn.pending(sn.getStatus);
+        auto pending = sn.pendingGet();
         if (sn.last_get_reply < std::max(now - Node::NODE_EXPIRE_TIME, last_get) or pending) {
             // not isSynced
             if (not pending and solicited_nodes < MAX_REQUESTED_SEARCH_NODES)
