@@ -29,9 +29,9 @@ extern "C" {
 using namespace dht;
 
 void print_usage() {
-    std::cout << "Usage: dhtnode [-v [-l logfile]] [-i] [-d] [-n network_id] [-p local_port] [-b bootstrap_host[:port]]" << std::endl << std::endl;
+    std::cout << "Usage: dhtnode [-v [-l logfile]] [-i] [-d] [-n network_id] [-p local_port] [-b bootstrap_host[:port]] [--proxyserver local_port]" << std::endl << std::endl;
     std::cout << "dhtnode, a simple OpenDHT command line node runner." << std::endl;
-    std::cout << "Report bugs to: http://opendht.net" << std::endl;
+    std::cout << "Report bugs to: https://opendht.net" << std::endl;
 }
 
 void print_id_req() {
@@ -55,15 +55,23 @@ void print_help() {
               << "  ll         Print basic information and stats about the current node." << std::endl
               << "  ls [key]   Print basic information about current search(es)." << std::endl
               << "  ld [key]   Print basic information about currenty stored values on this node (or key)." << std::endl
-              << "  lr         Print the full current routing table of this node" << std::endl;
+              << "  lr         Print the full current routing table of this node." << std::endl;
+
+#if OPENDHT_PROXY_SERVER
+    std::cout << std::endl << "Operations with the proxy:" << std::endl
+              << "  pst [port]            Start the proxy interface on port." << std::endl
+              << "  psp [port]            Stop the proxy interface on port." << std::endl;
+#endif //OPENDHT_PROXY_SERVER
 
     std::cout << std::endl << "Operations on the DHT:" << std::endl
-              << "  b <ip:port>             Ping potential node at given IP address/port." << std::endl
+              << "  b <ip:port>           Ping potential node at given IP address/port." << std::endl
               << "  g <key>               Get values at <key>." << std::endl
               << "  l <key>               Listen for value changes at <key>." << std::endl
               << "  p <key> <str>         Put string value at <key>." << std::endl
+              << "  pp <key> <str>        Put string value at <key> (persistent version)." << std::endl
               << "  s <key> <str>         Put string value at <key>, signed with our generated private key." << std::endl
               << "  e <key> <dest> <str>  Put string value at <key>, encrypted for <dest> with its public key (if found)." << std::endl;
+
     std::cout << std::endl << "Indexation operations on the DHT:" << std::endl
               << "  il <name> <key> [exact match]   Lookup the index named <name> with the key <key>." << std::endl
               << "                                  Set [exact match] to 'false' for inexact match lookup." << std::endl
@@ -82,6 +90,12 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
 #endif
 
     std::map<std::string, indexation::Pht> indexes;
+#if OPENDHT_PROXY_SERVER
+    std::map<in_port_t, std::unique_ptr<DhtProxyServer>> proxies;
+    if (params.proxyserver != 0) {
+        proxies.emplace(params.proxyserver, new DhtProxyServer(dht, params.proxyserver));
+    }
+#endif //OPENDHT_PROXY_SERVER
 
     while (true)
     {
@@ -115,20 +129,18 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
         } else if (op == "ld") {
             iss >> idstr;
             InfoHash filter(idstr);
-            if (filter == InfoHash{})
-                std::cout << dht->getStorageLog() << std::endl;
-            else
+            if (filter)
                 std::cout << dht->getStorageLog(filter) << std::endl;
+            else
+                std::cout << dht->getStorageLog() << std::endl;
             continue;
         } else if (op == "ls") {
             iss >> idstr;
             InfoHash filter(idstr);
-            if (filter == InfoHash{}) {
-                std::cout << "Searches:" << std::endl;
-                std::cout << dht->getSearchesLog() << std::endl;
-            } else {
+            if (filter)
                 std::cout << dht->getSearchLog(filter) << std::endl;
-            }
+            else
+                std::cout << dht->getSearchesLog() << std::endl;
             continue;
         } else if (op == "la")  {
             std::cout << "Reported public addresses:" << std::endl;
@@ -161,16 +173,35 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
             dht->setLogFilter(filter);
             continue;
         }
+#if OPENDHT_PROXY_SERVER
+        else if (op == "pst") {
+            iss >> idstr;
+            try {
+                unsigned int port = std::stoi(idstr);
+                proxies.emplace(port, new DhtProxyServer(dht, port));
+            } catch (...) { }
+            continue;
+        } else if (op == "psp") {
+            iss >> idstr;
+            try {
+                auto it = proxies.find(std::stoi(idstr));
+                if (it != proxies.end())
+                    proxies.erase(it);
+            } catch (...) { }
+            continue;
+        }
+#endif //OPENDHT_PROXY_SERVER
 
         if (op.empty())
             continue;
 
-        static const std::set<std::string> VALID_OPS {"g", "l", "il", "ii", "p", "s", "e", "a"};
+        static const std::set<std::string> VALID_OPS {"g", "l", "cl", "il", "ii", "p", "pp", "cpp", "s", "e", "a",  "q"};
         if (VALID_OPS.find(op) == VALID_OPS.cend()) {
             std::cout << "Unknown command: " << op << std::endl;
             std::cout << " (type 'h' or 'help' for a list of possible commands)" << std::endl;
             continue;
         }
+        dht::InfoHash id;
 
         if (op == "il" or op == "ii") {
             // Pht syntax
@@ -197,12 +228,17 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
         else {
             // Dht syntax
             iss >> idstr;
-            InfoHash h {idstr};
-            if (not isInfoHash(h))
-                continue;
+            id = dht::InfoHash(idstr);
+            if (not id) {
+                if (idstr.empty()) {
+                    std::cerr << "Syntax error: invalid InfoHash." << std::endl;
+                    continue;
+                }
+                id = InfoHash::get(idstr);
+                std::cout << "Using h(" << idstr << ") = " << id << std::endl;
+            }
         }
 
-        dht::InfoHash id {idstr};
         // Dht
         auto start = std::chrono::high_resolution_clock::now();
         if (op == "g") {
@@ -236,11 +272,17 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
         else if (op == "l") {
             std::string rem;
             std::getline(iss, rem);
-            dht->listen(id, [](std::shared_ptr<Value> value) {
+            auto token = dht->listen(id, [](std::shared_ptr<Value> value) {
                 std::cout << "Listen: found value:" << std::endl;
                 std::cout << "\t" << *value << std::endl;
                 return true;
             }, {}, dht::Where {std::move(rem)});
+            std::cout << "Listening, token: " << token.get() << std::endl;
+        }
+        else if (op == "cl") {
+            std::string rem;
+            iss >> rem;
+            dht->cancelListen(id, std::stoul(rem));
         }
         else if (op == "p") {
             std::string v;
@@ -253,6 +295,25 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
                 std::cout << "Put: " << (ok ? "success" : "failure") << " (took " << print_dt(end-start) << "s)" << std::endl;
             });
         }
+        else if (op == "pp") {
+            std::string v;
+            iss >> v;
+            auto value = std::make_shared<dht::Value>(
+                dht::ValueType::USER_DATA.id,
+                std::vector<uint8_t> {v.begin(), v.end()}
+            );
+            dht->put(id, value, [start,value](bool ok) {
+                auto end = std::chrono::high_resolution_clock::now();
+                auto flags(std::cout.flags());
+                std::cout << "Put: " << (ok ? "success" : "failure") << " (took " << print_dt(end-start) << "s). Value ID: " << std::hex << value->id << std::endl;
+                std::cout.flags(flags);
+            }, time_point::max(), true);
+        }
+        else if (op == "cpp") {
+            std::string rem;
+            iss >> rem;
+            dht->cancelPut(id, std::stoul(rem, nullptr, 16));
+        }
         else if (op == "s") {
             if (not params.generate_identity) {
                 print_id_req();
@@ -365,8 +426,8 @@ main(int argc, char **argv)
 
         dht::crypto::Identity crt {};
         if (params.generate_identity) {
-            auto ca_tmp = dht::crypto::generateIdentity("DHT Node CA");
-            crt = dht::crypto::generateIdentity("DHT Node", ca_tmp);
+            auto ca_tmp = dht::crypto::generateEcIdentity("DHT Node CA");
+            crt = dht::crypto::generateEcIdentity("DHT Node", ca_tmp);
         }
 
         dht->run(params.port, crt, true, params.network);
